@@ -582,15 +582,15 @@ QcdmResult *qcdm_cmd_nw_subsys_modem_snapshot_cdma_result (const char *buf,
 /* Values for QCDM_CMD_WCDMA_SUBSYS_STATE_INFO_ITEM_L1_STATE */
 
 enum {
-    QCDM_WCDMA_L1_STATE_IDLE = 0,
-    QCDM_WCDMA_L1_STATE_FS = 1,
-    QCDM_WCDMA_L1_STATE_ACQ = 2,
-    QCDM_WCDMA_L1_STATE_BCH = 3,
-    QCDM_WCDMA_L1_STATE_PCH = 4,
-    QCDM_WCDMA_L1_STATE_FACH = 5,
-    QCDM_WCDMA_L1_STATE_DCH = 6,
-    QCDM_WCDMA_L1_STATE_DEACTIVATED = 7,
-    QCDM_WCDMA_L1_STATE_PCH_SLEEP = 8,
+    QCDM_WCDMA_L1_STATE_INIT = 0,
+    QCDM_WCDMA_L1_STATE_IDLE = 1,
+    QCDM_WCDMA_L1_STATE_FS = 2,
+    QCDM_WCDMA_L1_STATE_ACQ = 3,
+    QCDM_WCDMA_L1_STATE_BCH = 4,
+    QCDM_WCDMA_L1_STATE_PCH = 5,
+    QCDM_WCDMA_L1_STATE_FACH = 6,
+    QCDM_WCDMA_L1_STATE_DCH = 7,
+    QCDM_WCDMA_L1_STATE_DEACTIVATE = 8,
     QCDM_WCDMA_L1_STATE_DEEP_SLEEP = 9,
     QCDM_WCDMA_L1_STATE_STOPPED = 10,
     QCDM_WCDMA_L1_STATE_SUSPENDED = 11,
